from bzv.infra import Infra
from bzv.customerConfig import CustomerConfig
from bzv.configWatcher import ConfigWatcher
from bzv.inbox import Inbox
from bzv.server import main, optimize
from bzv.observablehttpserver import BZVObservableHttpServer
from merescocore.framework import be
from weightless import Reactor
from sys import stdout, argv
from fcntl import flock, LOCK_EX, LOCK_NB
from os import environ, open, O_RDONLY, O_CREAT
from os.path import join as join_path
import sys, traceback
import syslog

environ['LANG']='en_US.UTF-8'

# Onderschep 'buitenste' exception en toon deze in syslog (zie: http://docs.python.org/library/sys.html)
def hooked(type, value, tb):
    syslog.syslog('meresco-server: FATAL')
    dump = traceback.format_exception(type, value, tb)
    for  line in dump:
        syslog.syslog(line)

	# print ook naar stderror:
    traceback.print_exception(type, value, tb)

sys.excepthook = hooked;

args = argv[1:]
if len(args) == 1:
	bzvConfigFile = args[0]

# Infra needs a logfile at startup
infra = Infra('Starting MerescoServer',configFile=bzvConfigFile)

configRoot = infra.configRoot

#fixme, alles nog ff bijelkaar geveegd wat eerst in bzv.cfg stond	
config = configRoot.getSectionDict('system')

# make sure no two instances access the same database
databasePath = config['databasepath']
lockFile = join_path(databasePath, 'lock')
fd = open(lockFile, O_RDONLY|O_CREAT)
flock(fd, LOCK_EX|LOCK_NB)

optimize(databasePath)

baseURL = config['baseurl']
portNumber = int(config['portnumber'])
sruUpdatePortNumber = int(config['sruupdateportnumber'])

# Expect all config files here
configWatcherDirectory = infra.configpath

logging = infra.logging
logger = logging.getLogger('meresco.server')
logging.debug('Starting meresco-server logging')
logger.debug('Starting meresco-server logger')

print 'Firing up Tilburg Beter Zoeken & Vinden Server.'
reactor = Reactor()
observableHttpServer = BZVObservableHttpServer(reactor, portNumber, infra = infra )
observableHttpServer.startServer()
if portNumber < 1024:
	runAs(config['username'], config['groupname'])

sruUpdateServer = BZVObservableHttpServer(reactor, sruUpdatePortNumber, prio = 9, infra = infra  )
sruUpdateServer.startServer()

inbox = Inbox(reactor=reactor, inboxDirectory=config['inboxdirectory'],
	      doneDirectory=config['donedirectory'], infra = infra)

configWatcher = ConfigWatcher(reactor=reactor,
			      infra = infra,
			      )

server = be(main(reactor, observableHttpServer, sruUpdateServer, inbox,
		 baseURL, portNumber, databasePath, infra, configWatcher))

server.once.observer_init()

inbox.processInboxDirectory()

print "Ready to rumble on", baseURL, "at ports", portNumber, sruUpdatePortNumber
print "   - database:", databasePath, "\n"
stdout.flush()
reactor.loop()
